
#if Q has reduced base locus, this returns those quatroids
#  which induce th
function Modifications(Q;verbose=false,override=false)
	signatures=QuatroidSignatures()
	SBL =[]
	if ReducedBaseLocus(Q)==true || override==true
		M = []
		for q in Q
			if length(q)==3
				push!(M,q)
				for q2 in Q
					#nonintersecting lines produce quadrics
					if length(q2)==3 && length(q∩q2)==0
						push!(M,vcat(filter(x->x∉q && x∉q2,collect(1:8)),[9]))
					end
				end
			elseif length(q)==6
				push!(M,vcat(filter(x->x∉q,collect(1:8)),[9]))
			end
		end
		if verbose
			println(M)
		end
		for i in 1:9
			Qnew = [] #we now delete each pt from the 9 to produce new quatroids
			for m in M
				if i ∈ m #if the deleted point is on the line, we get a quadric
					#which is a conic provided no other element of m is contained
					#in it
					conic=filter(j->j∉m,collect(1:9))
					if length(filter(x->issubset(x,conic),M))==0
						relabeled = []
						for j in conic
							if j==9
								push!(relabeled,i)
							else
								push!(relabeled,j)
							end
						end
						relabeled=sort(relabeled)
						push!(Qnew,relabeled)
					end
				else
					relabeled=[]
					for j in m
						if j==9
							push!(relabeled,i)
						else
							push!(relabeled,j)
						end
					end
					push!(Qnew,sort(relabeled))
				end
			end
			Qnew = unique(Qnew)	#we may have doubled lines which are disjoint
			push!(SBL,QuatroidNumber(Qnew))
		end
		return(sort(unique(vcat(SBL...))))
	else
		return([QuatroidNumber(Q)])
	end
end


function QuatroidReduction(i;verbose=false)
	SDs=ShortDescriptions()
	QM = QuatroidModifications()

	SD = SDs[i]
	Lines = SD[1]
	Conics = SD[2]

	ReductionFound=false
	Reason = []
	ReducedTo = nothing

	if length(Lines)<= 6 && length(Conics)==0
		if verbose
			println("Six or fewer lines")
		end
		Reason = [3,nothing]
		ReducedTo = 1
		ReductionFound=true
	else
		for j in 1:8
			if ReductionFound==false
				LinesContainingPoint = filter(x->j ∈ x,Lines)
				ConicsContainingPoint = filter(x->j ∈ x, Conics)
				if verbose
					println(j)
					println(LinesContainingPoint)
					println(ConicsContainingPoint)
				end
				if length(LinesContainingPoint)∈[1,2] && length(ConicsContainingPoint)==0
					Reason = [1,j]
					ReductionFound=true
				end
				if length(LinesContainingPoint)==0 && length(ConicsContainingPoint)==1
					Reason = [2,j]
					ReductionFound=true
				end
			end
		end
	end

	if ReductionFound==false
		if QM[i]!=[[i]]
			Reason = [4,nothing]
			ReducedTo = i
		else
			Reason = [0,nothing]
			ReducedTo = i
		end
	elseif Reason[1]<3
		deletable = Reason[2]
		NewLines = filter(x->deletable ∉ x,Lines)
		NewConics = filter(x->deletable ∉ x,Conics)
		ReducedTo = QuatroidNumber([NewLines,NewConics];sd=true)
	end
	return([Reason,ReducedTo])
end


function ParseReductionData(ReductionDictionary,ReasonDictionary;verbose=false)
	FinalReductions = []
	FullReasons = []
	for i in 1:126
		current = i
		new = nothing
		ReasonString = string(i)
		done = false
		while done == false
			new = ReductionDictionary[current]
			reason = ReasonDictionary[current]
			if reason[1]==1
				ReasonString=ReasonString*"--(i: delete "*string(reason[2])*")-->"
				ReasonString=ReasonString*string(new)
			end

			if reason[1]==2
				ReasonString=ReasonString*"--(ii: delete "*string(reason[2])*")-->"
				ReasonString=ReasonString*string(new)
			end

			if reason[1]==3
				ReasonString=ReasonString*"--(iii <= 6 lines)-->"
				ReasonString=ReasonString*string(new)
			end
			if new == current
				done=true
			else
				current=new
			end
		end
		push!(FinalReductions,new)
		push!(FullReasons,ReasonString)
	end
	Result = [FinalReductions,FullReasons]
	ToBeReduced = findall(x->FinalReductions[x]!=1,1:126)
	if verbose
		println("The first three types of reductions show all are irreducible except ")
		println(ToBeReduced)
	end
	CompleteResult = CompleteReductionParse(Result,ReductionDictionary,ReasonDictionary;verbose=verbose)
	return(CompleteResult)
end

function CompleteReductionParse(Result,ReductionDictionary,ReasonDictionary;verbose=false)
	Reductions = Result[1]
	Reasons = Result[2]
	AllQuatroids = GenerateAllCandidateQuatroids()
	for i in 1:126
		if i!=63
			if ReasonDictionary[i][1]==4 #if there are modifications
				#find any which have been reduced
				Mod = Modifications(AllQuatroids[i])
				if verbose
					println("Checking modifications for quatroid ",i)
				end
				loc = findall(x->Reductions[x]==1,Mod)
				if length(loc)>0 #if any have been reduced
					newQindex = Mod[loc[1]]
					Reasons[i]=Reasons[i]*"--(iv,"*string(newQindex)*")-->"
					Reasons[i]=Reasons[i]*Reasons[newQindex]
					Reductions[i]=Reductions[newQindex]
					ReductionDictionary[i]=newQindex
					ReasonDictionary[i]=[4,nothing]
				else
					if length(Mod)>1
						Reasons[i]=Reasons[i]*"--(iv,0)-->"*string(Mod)
						Reductions[i]=filter(x->x!=i,Mod)[1]
						ReductionDictionary[i] = filter(x->x!=i,Mod)[1]
						ReasonDictionary[i]=[4,nothing]
					end
				end
			end
		end
	end
	return([Reductions,Reasons])
end
